#ifndef GENERALTREE_H
#define GENERALTREE_H

#include "LinkedList.h"

template <class dataType>
class GeneralTree
{
public:
// ----------------------------------------------------------------
//  Name:           GeneralTree::GeneralTree
//  Description:    Default Constructor. Creates a tree node.
//  Arguments:      - data: data to initialise node with
//  Return Value:   None.
// ----------------------------------------------------------------
    GeneralTree( dataType data )
        : m_data( data ) {};


// ----------------------------------------------------------------
//  Name:           GeneralTree::~GeneralTree
//  Description:    Destructor. Deletes all child nodes.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    ~GeneralTree()
    {
        // delete each child. Since the destructor is recursive, 
        // each child node we delete automatically deletes all
        // of it's children. Neat, huh?
        start();
        while( isValid() )
        {
            delete m_children.item();
            forth();
        }
    }


// ----------------------------------------------------------------
//  Name:           GeneralTree::addChild
//  Description:    Adds a child node to the tree's child list.
//  Arguments:      - tree: sub-tree to add
//  Return Value:   None.
// ----------------------------------------------------------------
    void addChild( GeneralTree<dataType>* tree )
    {
        m_children.append( tree );
    }


// ----------------------------------------------------------------
//  Name:           GeneralTree::start
//  Description:    resets the child iterator to point to the first
//                  child in the current tree.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void start()
    {
        m_children.start();
    }


// ----------------------------------------------------------------
//  Name:           GeneralTree::forth
//  Description:    moves the child iterator forward to point to
//                  the next child.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void forth()
    {
        m_children.forth();
    }


// ----------------------------------------------------------------
//  Name:           GeneralTree::isValid
//  Description:    determines if the child iterator points to a
//                  valid child.
//  Arguments:      None.
//  Return Value:   True if the iterator points to a valid child,
//                  False otherwise.
// ----------------------------------------------------------------
    bool isValid()
    {
        return m_children.isValid();
    }


// ----------------------------------------------------------------
//  Name:           GeneralTree::child
//  Description:    returns a pointer to the current child tree
//                  that the child iterator points to.
//  Arguments:      None.
//  Return Value:   pointer to current child tree
// ----------------------------------------------------------------
    GeneralTree<dataType>* child()
    {
        return m_children.item();
    };


// ----------------------------------------------------------------
//  Name:           GeneralTree::removeChild
//  Description:    removes the child that the child iterator
//                  is pointing tom and moves child iterator to
//                  next child.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void removeChild()
    {
        m_children.remove();
    };


// ----------------------------------------------------------------
//  Name:           GeneralTree::preorder
//  Description:    processes the list using a pre-order traversal
//  Arguments:      - process: A function which takes a reference
//                             to a dataType and performs an
//                             operation on it.
//  Return Value:   None.
// ----------------------------------------------------------------
    void preorder( void (*process)(dataType& item) )
    {
        process( m_data );
        start();
        while( isValid() )
        {
            child()->preorder( process );
            forth();
        }
    }

// ----------------------------------------------------------------
//  Name:           GeneralTree::postorder
//  Description:    processes the list using a post-order traversal
//  Arguments:      - process: A function which takes a reference
//                             to a dataType and performs an
//                             operation on it.
//  Return Value:   None.
// ----------------------------------------------------------------
    void postorder( void (*process)(dataType& item) )
    {
        start();
        while( isValid() )
        {
            child()->postorder( process );
            forth();
        }
        process( m_data );
    }



protected:
// ----------------------------------------------------------------
//  Name:           GeneralTree::m_data
//  Description:    data stored in current tree node
// ----------------------------------------------------------------
    dataType m_data;

// ----------------------------------------------------------------
//  Name:           GeneralTree::m_children
//  Description:    children list
// ----------------------------------------------------------------
    LinkedList<GeneralTree<dataType>*> m_children;

}; 

#endif
